/**
 * @file exceptionBase.cpp
 *
 * @author Jan Hadrava
 * @author Petr Cermak
 *
 * @ingroup iopccommon
 */

#include "exceptionBase.h"
#include <string>

using namespace std;

namespace iopc {

IopcException::IopcException(const string& msg = "")
	throw ()
{
	message = name + ": " + msg;
}

IopcException::IopcException(const std::string& exceptionName, const std::string& msg)
		throw()
{
	message = exceptionName + ": " + msg;
}

IopcException::~IopcException()
	throw()
{

}

const char* IopcException::what() const
	throw()
{
	return message.c_str();
}

IopcNotSupportedException::IopcNotSupportedException(const string& msg)
	throw ()
	: IopcException(IopcNotSupportedException::name, msg)
{
}

IopcNoMemoryException::IopcNoMemoryException(const string& msg)
	throw ()
	: IopcException(IopcNoMemoryException::name, msg)
{
}

IopcIncorrectUsageException::IopcIncorrectUsageException(const string& msg)
	throw ()
	: IopcException(IopcIncorrectUsageException::name, msg)
{
}

IopcInvalidStateException::IopcInvalidStateException(const string& msg)
	throw ()
	: IopcException(IopcInvalidStateException::name, msg)
{
}

IopcRsrcLackException::IopcRsrcLackException(const string& msg)
	throw ()
	: IopcException(IopcRsrcLackException::name, msg)
{
}

IopcDeadlockException::IopcDeadlockException(const string& msg)
	throw ()
	: IopcException(IopcDeadlockException::name, msg)
{
}

IopcDbException::IopcDbException(const string& msg)
	throw ()
	: IopcException(IopcDbException::name, msg)
{
}

IopcDbException::IopcDbException(const string& exceptionName, const string& msg)
	throw()
	: IopcException(exceptionName, msg)
{
}

IopcDbDeadlockException::IopcDbDeadlockException(const string& msg)
	throw ()
	: IopcDbException(IopcDbDeadlockException::name, msg)
{
}

IopcDuplicateKeyException::IopcDuplicateKeyException(const string& msg)
	throw()
	: IopcException(IopcDuplicateKeyException::name, msg)
{
}

IopcNotFoundException::IopcNotFoundException(const string& msg)
	throw()
	: IopcException(IopcNotFoundException::name, msg)
{
}

IopcMetadataException::IopcMetadataException(const string& metadata, const string& location, const string& msg)
	throw()
	: IopcException(IopcMetadataException::name, "Metadadata: " + metadata + "; location: " + location + "; error: " + msg)
{
}

IopcIOException::IopcIOException(const string& msg)
	throw ()
	: IopcException(IopcIOException::name, msg)
{
}

IopcNoRowsFetchedException::IopcNoRowsFetchedException(const string& msg)
	throw ()
	: IopcException(IopcNoRowsFetchedException::name, msg)
{
}

const string IopcException::name = "General IOPC exception";
const string IopcNotSupportedException::name = "Required functionality is not supported";
const string IopcNoMemoryException::name = "Out of memory";
const string IopcIncorrectUsageException::name = "Incorrect usage of IOPC interface";
const string IopcInvalidStateException::name = "The object is in an invalid state";
const string IopcDeadlockException::name = "One-thread deadlock detected";
const string IopcRsrcLackException::name = "Lack of system resources or unsufficient privileges";
const string IopcDbException::name = "Database exception";
const string IopcDbDeadlockException::name = "Deadlock in database exception";
const string IopcDuplicateKeyException::name = "Attempt to insert duplicate key into a collection";
const string IopcNotFoundException::name = "Not found exception";
const string IopcMetadataException::name = "Metadata error";
const string IopcIOException::name = "IO exception";
const string IopcNoRowsFetchedException::name = "No rows fetched";
}
