/**
 * @file oracle10gDriver.cpp
 * 
 * @author Petr Cermak
 */


#include "oracle10gDriver.h"
#include "oracle10gDatabaseImpl.h"
#include "iopcdb/database.h"
#include "iopccommon/globalSettings.h"
#include <string>

#include "iopcdb/driverManager.h"

namespace iopc {
	
//bool Oracle10gDriver::initialized = false;

Oracle10gDriver::Oracle10gDriver(const std::string& driverName)
	: Driver(driverName)//, objectMappingFeature(sqlStatementsFeature, typeMappingFeature)
{
}

DatabaseImpl* Oracle10gDriver::getDatabaseImpl(const std::string& dbName)
{
	return new Oracle10gDatabaseImpl(dbName);
}

void Oracle10gDriver::returnDatabaseImpl(DatabaseImpl* db)
{
	delete db;
}

void Oracle10gDriver::doInit()
{
	if (!globalSettings.hasMeta("db.oracle.oidSequence.name")) {
		globalSettings["db.oracle.oidSequence.name"].setStringValue("OIDSEQ");
	}
	if (!globalSettings.hasMeta("db.oracle.serialIdSequence.name")) {
		globalSettings["db.oracle.serialIdSequence.name"].setStringValue("SERIALSEQ");
	}
	
	registerFeature<SqlStatementsFeature>(sqlStatementsFeature);
	//registerFeature<ObjectMappingFeature>(objectMappingFeature);
	registerFeature<TypeMappingFeature>(typeMappingFeature);
	
	typeMappingFeature.initTypes();
}


static Oracle10gDriver oracle10gDriverInstance("IopcOracle10g");


}
