#ifndef ORACLE10GORSTATEMENTSFEATURE_H_
#define ORACLE10GORSTATEMENTSFEATURE_H_

#include "iopclib/db/orStatementsFeature.h"
#include "iopcdriver_oracle10g/oracle10gSqlStatementsFeature.h"
#include "iopcdriver_oracle10g/oracle10gTypeMappingFeature.h"
#include "iopclib/types/dbType.h"
#include "iopclib/types/orMappingStrategy.h"
#include "iopclib/db/statementCache.h"
#include <sstream>
#include <set>
#include <string>

namespace iopc {

class Type;

class Oracle10gORStatementsFeature : public ORStatementsFeature
{	
private:
	Oracle10gSqlStatementsFeature* sqlStatements;
	Oracle10gTypeMappingFeature* typeMapping;
	
	std::string getWhere(const DbType::KeyAttributesSet& keys, std::string metaColName);
	
	StatementCache& cache;
public:
	Oracle10gORStatementsFeature(StatementCache& cache) : cache(cache) {};

	inline void init(SqlStatementsFeature& sqlStatements,
		TypeMappingFeature& typeMapping) {
		this->sqlStatements = dynamic_cast<Oracle10gSqlStatementsFeature*>(&sqlStatements);
		this->typeMapping = dynamic_cast<Oracle10gTypeMappingFeature*>(&typeMapping);
	}

	virtual std::string getInsertStatement(ORMappingStrategy& currentStrategy, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getSelectFromClassViewStatement(ORMappingStrategy& currentStrategy, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getSelectFromClassTableStatement(ORMappingStrategy& currentStrategy, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getUpdateStatement(ORMappingStrategy& currentStrategy, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getUpdateStatement(ORMappingStrategy& currentStrategy, const ORMappingStrategy::MappedAttributesList& updateCols, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getDeleteStatement(ORMappingStrategy& currentStrategy, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual bool validate(ORMappingStrategy& currentStrategy, std::string& msg);
	
	std::vector<std::string> getCreateClassTableScript(ORMappingStrategy& currentStrategy,  const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	std::vector<std::string> getDropClassTableScript(ORMappingStrategy& currentStrategy,  const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	void processAttributesForView(ORMappingStrategy& currentStrategy, bool& start, std::ostringstream& ocolsStr, std::set<std::string>& processedColumns, const Type& mainType);
};


}

#endif /*ORACLE10GORSTATEMENTSFEATURE_H_*/
