#ifndef COMPOSEDCACHE_H_
#define COMPOSEDCACHE_H_

// project: POLiTe
// authors: Jan Hadrava 2004

/** \file */
#include "iopcdb/connection.h"
#include "cache.h"
#include "strategySelector.h"
#include "cacheSelector.h"
#include "iopccommon/defs.h"

namespace iopc {

/** Basic cache composed of several extended caches. */
class DLL_EXTERNAL ComposedCache: public Cache {
public:
	ComposedCache();
	ComposedCache(CacheSelector* cacheSel, StrategySelector* stratSel);
	virtual ~ComposedCache();
	void setCacheSelector(CacheSelector* cacheSel);
	void setStrategySelector(StrategySelector* strategySel);
	virtual Cache* clone();
private:/*hiding*/
	ComposedCache(const ComposedCache& );
	ComposedCache& operator=(const ComposedCache& );
	
protected:	/* used by Connection */
	
	virtual void setConnection(Connection* conn = NULL);
	/* called in connection destructor -> can't throw exception */
	virtual void clear();

	virtual CacheLock& makePersistent(const PersIdentification& persId, DatabaseObject& objectPtr);
	virtual CacheLock& getCacheLock(const PersIdentification& persId, bool rdonly = false);

	virtual void dbDelete(const PersIdentification& persId);
	virtual void cacheDelete(const PersIdentification& persId);

	virtual void updateDirty();
	virtual void removeAll();
protected:
	CacheSelector* cacheSel;	/**< Provides access to subcaches. */
	StrategySelector* stratSel;	/**< Provides access to strategy settings. */
	
private:
	void checkState();
};

}/*namespace iopc*/



#endif /*COMPOSEDCACHE_H_*/
