#include "voidCacheItem.h"
#include "voidCache.h"
#include "iopcdb/connection.h"
#include "iopccommon/trace.h"

namespace iopc {

VoidCacheItem::VoidCacheItem(VoidCache& cache, const PersIdentification& persId, DatabaseObject& obj, bool isNew)
	: cache(cache), persId(persId)
{
	IOPC_FUNC_INVOKED;
	//this->cache = cache;
	//this->persId = persId;
	this->objectPtr = &obj;
	this->isNewItem = isNew;
}

VoidCacheItem::~VoidCacheItem()
{
	IOPC_FUNC_INVOKED;
	try {
		lock.xlock();	/* wait for all locks to be unlocked */
		lock.unlock();
		if (objectPtr != NULL) {
			delete objectPtr;
		}
	} catch (...) {
		IOPC_UNREACHABLE;
	}
}

DatabaseObject& VoidCacheItem::getObject()
{
	return *objectPtr;
}

CacheLock& VoidCacheItem::addLock(bool rdonly)
{
	if (rdonly) {
		lock.rlock();
	} else {
		lock.wlock();
	}
	return *this;
}

void VoidCacheItem::rmLock()
{
	IOPC_FUNC_INVOKED;
	lock.unlock();
	cache.tryCacheDelete(persId);	/* trying to commit indirect suicide */
	IOPC_FUNC_ENDED;
}

void VoidCacheItem::unlock()
{
	lock.unlock();
}

bool VoidCacheItem::isItemLocked()
{
	if (!lock.tryXLock()) {	/* item is locked */
		return true;
	}
	lock.unlock();
	return false;
}

}
