#ifndef OBJECTMAPPINGSTRATEGY_H_
#define OBJECTMAPPINGSTRATEGY_H_

#include "mappingStrategy.h"
#include "iopcmeta/type.h"
#include "dbType.h"
#include <set>
#include "dbTypeManager.h"

namespace iopc {

class ObjectMappingStrategy : public MappingStrategy
{
private:
	DbType* dbType;
public:
	ObjectMappingStrategy(DbType* dbType) : dbType(dbType) {};
	virtual void init();
	virtual void postInit();
	virtual void loadOne(Connection& conn, DatabaseObject* obj, const KeyValues& keyVals, bool wait, bool lock);
	virtual OidObject* viewLoadAll(Connection& conn, const OidType& oid, bool wait, bool lock);
	virtual void updateOne(Connection& conn, DatabaseObject& obj);
	virtual void deleteOne(Connection& conn, const OidType& oid);
	virtual void deleteOne(Connection& conn, const KeyValues& keyVals);
	virtual void insertOne(Connection& conn, DatabaseObject& obj);
	inline virtual DbType& getDbType() const { return *dbType; }
	inline virtual const Type& getType() const { return dbType->getType(); }
public:
	typedef std::set<const DbType*> RootTypesList;
	typedef RootTypesList::const_iterator RootTypesIterator;

	typedef std::vector<const Attribute*> PersistentAttributesList;
	typedef PersistentAttributesList::const_iterator PersistentAttributesIterator;
private:
	PersistentAttributesList attributes;
	PersistentAttributesList persistentAttributes;
	RootTypesList roots;
	bool root;

public:
	inline bool isRoot() const { return root; }
	inline const RootTypesList& getRootTypes() const { return roots; }
	inline const ObjectMappingStrategy& getFirstRoot() const { return (*roots.begin())->getObjectMappingStrategy(); }
	inline const PersistentAttributesList& getMappedAttributes() const { return attributes; }
	// Same as DbType.getPersistentAttributes, only doesn't contain the oid column even in the OidObject and its descendants
	inline const PersistentAttributesList& getPersistentAttributes() const { return persistentAttributes; }
	inline const Attribute& getOidColumn() const { return getType().getAttributeAll("oid"); }
	inline ObjectMappingStrategy& getFirstParentOMStrategy() const { return DbTypeManager::getDbTypeWithInit(getType().getFirstParent()).getObjectMappingStrategy(); }
};

}
#endif /*OBJECTMAPPINGSTRATEGY_H_*/
