#include "tests/oracle10gCursorTest.h"

#include "iopccommon/trace.h"
#include "iopcdb/database.h"
#include "iopcdb/connection.h"
#include "iopcdb/cursor.h"
#include "iopccommon/exceptionBase.h"
#include "iopcdb/driverManager.h"
#include "iopcmeta/typeDesc.h"
#include <iostream>

using namespace iopc;
using namespace std;


Oracle10gCursorTest::Oracle10gCursorTest(const std::string& testName)
 : TestCase(testName)
{

}

Oracle10gCursorTest::~Oracle10gCursorTest()
{
}

bool Oracle10gCursorTest::doTest()
{
	Database* db = NULL;
	Connection* conn = NULL;
	bool ret = true;
	try {
		db = DriverManager::getDriver("IopcOracle10g").getDatabase(DBNAME);
		conn = db->getConnection(CONNSTR);
		conn->open();
		conn->setAutoCommit(false);

		// prepare test
		try {
			conn->sqlNonQuery("drop table iopctest purge");
		} catch (...) {}

		// main test
		// create table
		conn->sqlNonQuery("create table iopctest (a varchar2(2) primary key)");

		// try to generate an error
		try {
			conn->sqlNonQuery("create table iopctest (a number primary key)");
			ret = false;
		} catch (std::exception& e) {
			logMsg(e.what());
		}
		// advanced cursor behavior

		conn->sqlNonQuery("insert into iopctest values('2')");
		conn->sqlNonQuery("insert into iopctest values('1')");
		conn->commit();
		conn->sqlNonQuery("insert into iopctest values('3')");
		conn->rollback();



		Cursor* cur = conn->sql("select a from iopctest order by a");
		cur->open();
		cur->prepare();
		cur->execute();

		int var;
		cur->preFetchBind(1, &var, sizeof(var), TypeDesc<int>::getType(), NULL);

		int i = 1;
		while (cur->fetchNext()) {
			if (var != i) {
				ret = false;
			}
			i++;
		}
		if (i != 3) {
			ret = false;
		}

		cur->close();
		conn->returnCursor(cur);

		// Try not to return cursors to conneciton
		cur = conn->sql("select 1 from dual");
		cur->open();
		cur->prepare();
		cur->execute();
		cur = conn->sql("select 2 from dual");

		// cleanup
		conn->sqlNonQuery("drop table iopctest purge");
		//conn->close();


	} catch (std::exception& e) {
		LogWriter::logError(e.what());
		ret = false;
	}

	/*if (conn) {
		db->returnConnection(conn);
	}*/

	try {
		if (db) DriverManager::getDriver("IopcOracle10g").returnDatabase(db);
	} catch (...) {
		ret = false;
	}
	return ret;
}



static Oracle10gCursorTest oracle10gCursorTestInstance("Oracle10g - Cursor");
